@@ -181,6 +181,41 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
       return(self$getTags())
     },
     
+    #'@description Download a metadata by UUID.
+    #'@param uuid uuid
+    #'@param addSchemaLocation add schema location. Default is \code{TRUE}
+    #'@param increasePopularity increase popularity. Default is \code{TRUE}
+    #'@param approved approved
+    #'@param filename output filename
+    downloadMetadataByUUID = function(uuid, 
+                                 addSchemaLocation = TRUE, increasePopularity = TRUE, approved = TRUE,
+                                 filename){
+      addSchemaLocation <- tolower(as.character(addSchemaLocation))
+      increasePopularity <- tolower(as.character(increasePopularity))
+      approved <- tolower(as.character(approved))
+      
+      self$INFO(sprintf("Fetching metadata for uuid = '%s'", uuid))
+      out <- NULL
+      req <- GNUtils$GET(
+        url = self$getUrl(),
+        path = sprintf("/api/records/%s/formatters/xml?addSchemaLocation=%s&increasePopularity=%s&approved=%s", 
+                       uuid, addSchemaLocation, increasePopularity, approved),
+        token = private$getToken(), cookies = private$cookies,
+        user = private$user,
+        pwd = private$getPwd(),
+        accept = "application/xml", contentType = "application/xml",
+        verbose = self$verbose.debug
+      )
+      if(status_code(req) == 200){
+        self$INFO("Successfully fetched metadata!")
+        xml <- GNUtils$parseResponseXML(req, "UTF-8")
+        XML::saveXML(xml, file = filename)
+      }else{
+        self$ERROR(sprintf("Error while fetching metadata - %s", message_for_status(status_code(req))))
+        self$ERROR(content(req))
+      }
+    },
+    
     #'@description Get a metadata by UUID.
     #'@param uuid uuid
     #'@param addSchemaLocation add schema location. Default is \code{TRUE}
@@ -236,8 +271,7 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
     
     #'@description Inserts a record by file, XML object or \pkg{geometa} object of class \link[geometa]{ISOMetadata} or \link[geometa]{ISOFeatureCatalogue}. 
     #'    Extra parameters related to \pkg{geometa} objects: \code{geometa_validate} (TRUE by default) and \code{geometa_inspire} 
-    #'    (FALSE by default) can be used to perform ISO and INSPIRE validation respectively. In that case on object of class 
-    #'    \code{geometa::INSPIREMetadataValidator}, with a proper user API key, should be specified as \code{geometa_inspireValidator} 
+    #'    (FALSE by default) can be used to perform ISO and INSPIRE validation respectively.
     #'    argument.
     #'@param xml object of class \link[XML]{XMLInternalNode-class}
     #'@param file file
@@ -253,7 +287,11 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
     #'@param extra extra
     #'@param geometa_validate validate geometa object
     #'@param geometa_inspire validate geometa object vs. INSPIRE
-    #'@param geometa_inspireValidator geometa INSPIRE validator to use 
+    #'@param geometa_inspireValidator geometa INSPIRE validator to use. Deprecated, see below note.
+    #'
+    #'@note From 2025-05-02, the INSPIRE metadata validation does not require anymore an API Key. Therefore, it is not
+    #'required to specify an \code{geometa_inspireValidator}. To send your metadata to INSPIRE, just set \code{geometa_inspire} 
+    #'to \code{TRUE}.
     insertRecord = function(xml = NULL, file = NULL, geometa = NULL,
                             metadataType = "METADATA", uuidProcessing = "NOTHING", 
                             group, category = NULL, rejectIfInvalid = FALSE, publishToAll = TRUE,
@@ -342,9 +380,7 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
     
     #'@description Inserts a metadata by file, XML object or \pkg{geometa} object of class \link[geometa]{ISOMetadata} or \link[geometa]{ISOFeatureCatalogue}. 
     #'    Extra parameters related to \pkg{geometa} objects: \code{geometa_validate} (TRUE by default) and \code{geometa_inspire} 
-    #'    (FALSE by default) can be used to perform ISO and INSPIRE validation respectively. In that case on object of class 
-    #'    \code{geometa::INSPIREMetadataValidator}, with a proper user API key, should be specified as
-    #'    \code{geometa_inspireValidator} argument.
+    #'    (FALSE by default) can be used to perform ISO and INSPIRE validation respectively.
     #'@param xml object of class \link[XML]{XMLInternalNode-class}
     #'@param file file
     #'@param geometa geometa object of class \link[geometa]{ISOMetadata} or \link[geometa]{ISOFeatureCatalogue}
@@ -359,7 +395,11 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
     #'@param extra extra
     #'@param geometa_validate validate geometa object
     #'@param geometa_inspire validate geometa object vs. INSPIRE
-    #'@param geometa_inspireValidator geometa INSPIRE validator to use 
+    #'@param geometa_inspireValidator geometa INSPIRE validator to use. Deprecated, see below note.
+    #'
+    #'@note From 2025-05-02, the INSPIRE metadata validation does not require anymore an API Key. Therefore, it is not
+    #'required to specify an \code{geometa_inspireValidator}. To send your metadata to INSPIRE, just set \code{geometa_inspire} 
+    #'to \code{TRUE}.
     insertMetadata = function(xml = NULL, file = NULL, geometa = NULL,
                               metadataType = "METADATA", uuidProcessing = "NOTHING", 
                               group, category = NULL, rejectIfInvalid = FALSE, publishToAll = TRUE,
@@ -376,9 +416,7 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
     
     #'@description Inserts a metadata by file, XML object or \pkg{geometa} object of class \link[geometa]{ISOMetadata} or \link[geometa]{ISOFeatureCatalogue}. 
     #'    Extra parameters related to \pkg{geometa} objects: \code{geometa_validate} (TRUE by default) and \code{geometa_inspire} 
-    #'    (FALSE by default) can be used to perform ISO and INSPIRE validation respectively. In that case on object of class 
-    #'    \code{geometa::INSPIREMetadataValidator}, with a proper user API key, should be specified as
-    #'    \code{geometa_inspireValidator} argument.
+    #'    (FALSE by default) can be used to perform ISO and INSPIRE validation respectively.
     #'@param xml object of class \link[XML]{XMLInternalNode-class}
     #'@param file file
     #'@param geometa geometa object of class \link[geometa]{ISOMetadata} or \link[geometa]{ISOFeatureCatalogue}
@@ -392,7 +430,11 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
     #'@param extra extra
     #'@param geometa_validate validate geometa object
     #'@param geometa_inspire validate geometa object vs. INSPIRE
-    #'@param geometa_inspireValidator geometa INSPIRE validator to use 
+    #'@param geometa_inspireValidator geometa INSPIRE validator to use. Deprecated, see below note.
+    #'
+    #'@note From 2025-05-02, the INSPIRE metadata validation does not require anymore an API Key. Therefore, it is not
+    #'required to specify an \code{geometa_inspireValidator}. To send your metadata to INSPIRE, just set \code{geometa_inspire} 
+    #'to \code{TRUE}.
     updateMetadata = function(xml = NULL, file = NULL, geometa = NULL,
                               metadataType = "METADATA",
                               group, category = NULL, rejectIfInvalid = FALSE, publishToAll = TRUE,
@@ -432,6 +474,81 @@ GNOpenAPIManager <- R6Class("GNOpenAPIManager",
       return(out)
     },
     
+    #' Set privilege configuration using Geonetwork OpenAPI
+    #'
+    #' @param id character; metadata Id
+    #' @param config object of class \link{GNPrivConfiguration} (same structure as legacy)
+    #' @return logical TRUE on success, FALSE otherwise
+    setPrivConfiguration = function(id, config){
+      self$INFO(sprintf("Setting privileges (OpenAPI) for metadata id = %s", id))
+      if(!is(config, "GNPrivConfiguration")){
+        stop("The 'config' value should be an object of class 'GNPrivConfiguration'")
+      }
+      
+      # 1) Fetch list of operations from server so we can provide a full operations map
+      reqOps <- GNUtils$GET(
+        url = self$getUrl(),
+        path = "/api/operations",
+        token = private$getToken(), cookies = private$cookies,
+        user = private$user,
+        pwd = private$getPwd(),
+        verbose = self$verbose.debug
+      )
+      
+      if(status_code(reqOps) != 200){
+        self$ERROR("Failed to fetch operations list")
+        self$ERROR(content(reqOps))
+        return(FALSE)
+      }
+      opsJson <- jsonlite::fromJSON(content(reqOps, as = "text", encoding = "UTF-8"))
+      print(dput(opsJson))
+      # opsJson is expected to be a list of objects with element 'name'
+      opIDs <- as.list(opsJson$id)
+      opNames <- as.list(opsJson$name)
+      
+      # 2) Build the privileges payload
+      # config$privileges is expected to be a list of grants where each grant has:
+      #  - group (group id, integer or string)
+      #  - privileges (character vector of operation names to grant)
+      privilegesList <- lapply(config$privileges, function(grant){
+        # For each known operation name, set TRUE if it's in grant$privileges, else FALSE
+        operationsMap <- setNames(
+          lapply(opIDs, function(op) { op %in% grant$privileges }),
+          opNames
+        )
+        # Ensure group is numeric / as in Geonetwork (could be -1, 0, 1 or custom ids)
+        list(
+          group = grant$group,
+          operations = operationsMap
+        )
+      })
+      
+      payload <- list(clear = TRUE, privileges = privilegesList)
+      payload_json <- jsonlite::toJSON(payload, auto_unbox = TRUE, pretty = FALSE)
+      
+      # 3) PUT payload to /srv/api/records/{id}/sharing
+      sharingPath <- paste0("/api/records/", id, "/sharing")
+      req <- GNUtils$PUT(
+        url = self$getUrl(),
+        path = sharingPath,
+        token = private$getToken(), cookies = private$cookies,
+        user = private$user,
+        pwd = private$getPwd(),
+        content = payload_json,
+        contentType = "application/json",
+        verbose = self$verbose.debug
+      )
+      
+      if(status_code(req) %in% c(200, 204)){
+        self$INFO(sprintf("Successfully set privileges for metadata ID = %s!", id))
+        return(TRUE)
+      }else{
+        self$ERROR(sprintf("Error while setting privileges (OpenAPI) - %s", message_for_status(status_code(req))))
+        self$ERROR(content(req))
+        return(FALSE)
+      }
+    },
+    
     #'@description Uploads attachment
     #'@param id metadata identifier
     #'@param file file to upload
